'use strict';

var Url = require('url');
var Util = require('util');
var Hoek = require('hoek');
var Wreck = require('wreck');
var GoodReporter = require('good-reporter');
var Mapper = require('./mapper');


var internals = {
    defaults: {
        database: 'good',
        username: undefined,
        password: undefined,
        threshold: 25,
        headers: {
            'Content-Type': 'application/octet-stream'
        }
    }
};


module.exports = internals.GoodInflux = function GoodInflux(host, options) {
    Hoek.assert(this instanceof internals.GoodInflux, 'GoodInflux must be created with new');
    Hoek.assert(typeof host === 'string', 'host must be a string');

    var settings, username, password, url, auth;

    settings = Hoek.clone(options || {});
    settings = Hoek.applyToDefaults(internals.defaults, settings);

    // Extract credentials from settings.
    username = settings.username;
    password = settings.password;

    // Try to parse credentials from url. (Overrides options.)
    url = Url.parse(host, true);
    if (typeof url.auth === 'string') {
        auth = url.auth.split(':');
        username = auth[0];
        password = auth[1];
        url.auth = null;
    }

    // Update settings
    settings.host = url.format();
    settings.username = undefined;
    settings.password = undefined;

    url.pathname = '/write';
    url.query = Util.format('?db=%s', settings.database);
    settings.url = Url.format(url);

    Hoek.assert(typeof username === 'string', 'username must be a string');
    Hoek.assert(typeof password === 'string', 'password must be a string');

    settings.headers.Authorization = 'Basic ' + new Buffer(username + ':' + password).toString('base64');

    GoodReporter.call(this, settings);
};


Hoek.inherits(internals.GoodInflux, GoodReporter);



internals.GoodInflux.prototype.start = function (emitter, callback) {
    emitter.on('report', this._handleEvent.bind(this));
    callback();
};


internals.GoodInflux.prototype.stop = function () {
    this._flush();
};


internals.GoodInflux.prototype._report = function (event, eventData) {
    this._eventQueue.push(eventData);

    if (this._eventQueue.length >= this._settings.threshold) {
        this._flush();
    }
};


internals.GoodInflux.prototype._flush = function () {
    var events, lines;

    events = this._eventQueue.splice(0);

    lines = Mapper.prepare(events);

    this._sendMessage(lines);
};


internals.GoodInflux.prototype._sendMessage = function (lines) {
    console.log('Lines: ' + lines);

    Wreck.request('post', this._settings.url + '?db=npmjs', { headers: this._settings.headers, payload: lines });

};
